import re

def match(pattern, path, exact=True, is_re=False, case_sensitive=False):
	"""
	The main motivation for keeping this function in a separate Python module,
	and not eg. as a method of Ignorable404Url, is so we can test it without any
	dependencies on Django (and in particular without having to initialize
	Django's settings just for a simple unit test).
	"""
	regex = pattern if is_re else re.escape(pattern)
	if exact:
		regex = '^%s/?$' % regex
	flags = 0 if case_sensitive else re.I
	return re.search(regex, path, flags)


# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2019-03-11 06:56
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Ignorable404Url',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('pattern', models.TextField(help_text='URLs matching this pattern are ignored.')),
                ('exact', models.BooleanField(default=True, verbose_name='The full URL must match')),
                ('is_re', models.BooleanField(default=False, verbose_name='Is regular expression')),
                ('case_sensitive', models.BooleanField(default=False, verbose_name='Is case sensitive')),
            ],
            options={
                'verbose_name': 'Ignorable 404 URL',
            },
        ),
    ]



from unittest import TestCase
from .match import match

class MatchTest(TestCase):
	def test_exact(self):
		self.assertTrue(match('/nonexistent', '/nonexistent'))
		self.assertFalse(match('/nonexistent', '/other-pattern'))
	def test_regex(self):
		self.assertTrue(match('/[0-9]+', '/1234', is_re=True))
		self.assertFalse(match('/[0-9]+', '/abc', is_re=True))
	def test_inexact(self):
		self.assertTrue(match('substr', '/some/substr/in/path', exact=False))
		self.assertFalse(match('substr', '/some/substr/in/path'))
	def test_case_sensitive(self):
		self.assertTrue(match('/foo', '/Foo'))
		self.assertFalse(match('/foo', '/Foo', case_sensitive=True))
	def test_inexact_regex(self):
		self.assertFalse(match('[0-9]', '/hi/3/there', is_re=True))
		self.assertTrue(match('[0-9]', '/hi/3/there', is_re=True, exact=False))


from django.contrib.admin import ModelAdmin, register
from django.contrib.admin.widgets import AdminTextInputWidget
from django.db.models import TextField

from .models import Ignorable404Url

@register(Ignorable404Url)
class Ignorable404UrlAdmin(ModelAdmin):
	formfield_overrides = {
		# Use a smaller text field:
		TextField: {'widget': AdminTextInputWidget},
	}


from django.conf import settings
from django.core.mail import mail_managers
from django.middleware.common import BrokenLinkEmailsMiddleware
from django.utils.encoding import force_text
from urllib.parse import urlencode

try:
	from django.urls import reverse
except ImportError:
	# Django < 2:
	from django.core.urlresolvers import reverse

class BrokenLinkEmailsDbMiddleware(BrokenLinkEmailsMiddleware):

	def process_response(self, request, response):
		"""
		Send broken link emails for relevant 404 NOT FOUND responses.
		"""
		if response.status_code == 404 and not settings.DEBUG:
			domain = request.get_host()
			path = request.get_full_path()
			referer = force_text(
				request.META.get('HTTP_REFERER', ''), errors='replace'
			)
			if not self.is_ignorable_request(request, path, domain, referer):
				subject = self._get_email_subject(domain, referer)
				message = self._get_email_message(request, path, referer)
				mail_managers(subject, message, fail_silently=True)
		return response

	def _get_email_subject(self, domain, referer):
		result = 'Broken '
		if self.is_internal_request(domain, referer):
			result += 'INTERNAL '
		result += 'link on ' + domain
		return result

	def _get_email_message(self, request, path, referer):
		user_agent = force_text(
			request.META.get('HTTP_USER_AGENT', '<none>'), errors='replace'
		)
		ip = request.META.get('REMOTE_ADDR', '<none>')
		result = 'Referrer: %s\n' % referer
		result += 'Requested URL: %s\n' % path
		result += 'User agent: %s\n' % user_agent
		result += 'IP address: %s' % ip

		admin_url = reverse('admin:django_404_middleware_ignorable404url_add')+\
				   '?' + urlencode({'pattern': path})
		result += '\n\nTo ignore this link, visit %s.' % \
				   request.build_absolute_uri(admin_url)
		return result

	def is_ignorable_request(self, request, uri, domain, referer):
		if super().is_ignorable_request(request, uri, domain, referer):
			return True
		# If we import this at module level, we get exception
		# AppRegistryNotReady when starting Django. So import it late, here:
		from .models import Ignorable404Url
		return any(i.matches(uri) for i in Ignorable404Url.objects.all())


"""An alternative to Django's BrokenLinkEmailsMiddleware

See:
https://github.com/mherrmann/django-404-middleware
"""

from setuptools import setup, find_packages

description = "An alternative to Django's BrokenLinkEmailsMiddleware"
url = 'https://github.com/mherrmann/django-404-middleware'
setup(
    name='django-404-middleware',
    version='0.0.2',
    description=description,
    long_description=description + '\n\nHome page: ' + url,
    author='Michael Herrmann',
    author_email='michael+removethisifyouarehuman@herrmann.io',
    url=url,
    packages=find_packages(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
    
        'License :: OSI Approved :: MIT License',
    
        'Operating System :: OS Independent',
    
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',

        'Framework :: Django',

        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    license='MIT',
    keywords='Django 404 Middleware Email',
    platforms=['MacOS', 'Windows', 'Debian', 'Fedora', 'CentOS', 'Arch'],
    test_suite='django_404_middleware.tests'
)


from django.db.models import Model, TextField, BooleanField
from django_404_middleware.match import match

class Ignorable404Url(Model):

	class Meta:
		verbose_name = 'Ignorable 404 URL'

	pattern = TextField(help_text='URLs matching this pattern are ignored.')
	exact = BooleanField(
		verbose_name='The full URL must match', blank=False, default=True
	)
	is_re = BooleanField(
		verbose_name='Is regular expression', blank=False, default=False
	)
	case_sensitive = BooleanField(
		verbose_name='Is case sensitive', blank=False, default=False
	)

	def matches(self, path):
		return bool(match(
			self.pattern, path, self.exact, self.is_re, self.case_sensitive
		))

	def __str__(self):
		return self.pattern